package {

	import net.flashpunk.*;
	import net.flashpunk.utils.*;
	import net.flashpunk.graphics.*;
	import flash.display.BitmapData;
	import flash.display.Graphics;

	/** STart up with a big mess to test a few things */

	public class Game extends World {
		public static const START: int = 0;
		public static const RUNNING: int = 1;
		public static const WIN: int = 2;

		
		public var level:Level;
		public var goals:Array;
		public var shapes:Array;
		public var collidable:Array;
		public var selected:Shape;
		public var direction: int;
		public var moves: int;
		public var state: int = 0;
	
		public var arrow: Entity;
		public var walls: Entity;
		public var moveText: Text;

		public var winOverlay: BitmapData;
		public var winText: Text;
		public var winButton: Button;
		public var winValue: Number = 0;
		public var previous: int;

		// Minimal editor ...
		private var edit:Object;
		private var editing:Boolean;


		public function Game(level:Level, prev:int=0) {
			FP.screen.color = 0xffffff;
			this.level = level;
			previous = prev;

			// Setup sidebar
			var side:Entity = createSidebar(100, String(level.index+1));
			side.graphic.scrollX = side.graphic.scrollY = 0;
			add(side);
			// Moves text
			var ms:Object = { size:16, width:100, height:100, align:"center", color:0x0, resizable:false, font:"brush" };
			moveText = new Text("Moves\n0 / " + level.target, 0, FP.height-200, ms);
			moveText.scrollX = moveText.scrollY=0;
			addGraphic(moveText).layer = -2;

			// Back button
			add( new Button("Back", 10,FP.height-50, 80, 40, goBack) ).layer = -2;
			// Reset button
			add( new Button("Reset", 10,FP.height-100, 80, 40, restart) ).layer = -2;

			

			// Turn sprite into bitmap for drawing
			var lvl:Stamp = level.wallGraphic();
			walls = addGraphic( lvl );
			walls.setHitboxTo(walls.graphic);


			// Add objects
			shapes = [];
			goals = [];
			collidable = [];
			for each(var e:Entity in level.objects) {
				add(e);
				collidable.push(e as GameObject);
				// Extract shapes
				if(e is Shape) shapes.push(e as Shape);
				// Extract goals
				if(e is Goal) goals.push(e as Goal);
			}
			
			// Sidebar 'complete' icons
			for(var i:int = 0; i<goals.length; ++i) {
				e = new Entity(30 + 40*(i&1), 150 + 40*(i>>1));
				e.graphic = new Graphiclist( goals[i].graphic );
				e.graphic.scrollX = e.graphic.scrollY = 0;
				add(e).layer = -2;
			}

			// Create move arrow
			arrow = addGraphic( createArrow(10, 25) );
			arrow.visible = false;

			// Centre camera
			var centreX: Number = side.width + (FP.width-side.width)/2;
			camera.x = walls.halfWidth + walls.graphic.x - centreX;
			camera.y = walls.halfHeight + walls.graphic.y - FP.halfHeight;
		}

		public static function createArrow(s:Number, l:Number): Image {
			var g:Graphics = FP.sprite.graphics;
			g.clear();
			g.lineStyle(1, 0x00, 1.0);
			g.moveTo(s,0);
			g.lineTo(2*s, s);
			g.lineTo(1.5*s, s);
			g.lineTo(1.5*s, l);
			g.lineTo(0.5*s, l);
			g.lineTo(0.5*s, s);
			g.lineTo(0, s);
			g.lineTo(s, 0);
			// Make image
			var bmp:BitmapData = new BitmapData(2*s+1, Math.max(l,s)+1, true, 0);
			bmp.draw(FP.sprite, null, null, null);
			var img:Image = new Image(bmp);
			img.originX = img.width/2;
			img.originY = img.height + Shape.radius + 5;
			return img;
		}

		public function createSidebar(w:Number, name:String): Entity {
			var g:Graphics = FP.sprite.graphics;
			g.clear();
			g.moveTo(0,0);
			g.beginFill(0xffffff, 1.0);
			g.lineTo(w+30, 0);
			g.curveTo(w,0, w,30);
			g.lineTo(w, FP.height-30);
			g.curveTo(w,FP.height, w+30,FP.height);
			g.lineTo(0,FP.height);
			g.lineTo(0,0);
			g.endFill();
			// Line
			g.lineStyle(4, 0x00, 1.0);
			g.moveTo(w+30, 0);
			g.curveTo(w,0, w,30);
			g.lineTo(w, FP.height-30);
			g.curveTo(w,FP.height, w+30,FP.height);
			// Create background
			var bmp:BitmapData = new BitmapData(w+32, FP.height, true, 0xffffff);
			bmp.draw(FP.sprite, null, null, null);
			var e:Entity = new Entity(0,0,new Stamp(bmp));
			// Level name
			var ts:Object = { width:w, height:100, align:"center", size:32, color:0x00, font:"brush" };
			Draw.setTarget(bmp);
			Draw.text("Level\n"+name, 0,10, ts);
			Draw.resetTarget();
			e.layer = -2;
			e.setHitbox(w, FP.height);
			return e;
		}

		/** Go back to previous menu */
		public function goBack(): void {
			Transition.fadeOut( previous? new LevelSelect(): new Title(), 0xffffff );
			Audio.play("change");
		}
		/** Restart level */
		public function restart(): void {
			Transition.fadeOut( new Game(level, previous), 0xffffff );
			Audio.play("change");
		}
		/** Start next level */
		public function nextLevel(): void {
			var next:Level = level.next;
			if(next) Transition.fadeOut( new Game(next, 1), 0xffffff );
			else Transition.fadeOut( new Ending(), 0xffffff );
			Audio.play("change");
		}

		/** Win Level */
		public function win(): void {
			state = WIN;
			level.score = level.score==0? moves: Math.min(level.score, moves);
			if(level.next) level.next.locked = false;
			level.saveData();
			Audio.play("win");
			GameStats.count(level.id, "win");
			GameStats.value(level.id, "moves", moves);
			GameStats.sendNow();

			// Fade overlay
			winOverlay= new BitmapData(walls.width, walls.height, true, 0xa0ffffff);
			var overlay:Stamp = new Stamp(winOverlay);
			addGraphic( overlay, -1, walls.graphic.x, walls.graphic.y );

			var cx:Number = 100 + (FP.width - 100)/2;
			
			// Complete text
			var cs:Object = { size:64, font:"brush", color:0x00 };
			winText = new Text("Complete!", 0, 0, cs);
			winText.scrollX = winText.scrollY = 0;
			addGraphic(winText, -1, cx - winText.textWidth/2, FP.halfHeight-winText.textHeight/2 - 60);

			// Continue button
			var py:Number = FP.halfHeight + winText.textHeight/2;
			winButton = new Button("Continue", cx-60, py, 120, 40, nextLevel);
			winButton.layer = -1;
			winButton.visible = false;
			add(winButton);
		}


		/** Begin game */
		public override function begin(): void {
			GameStats.count(level.id, "start");
			for each(var g:Goal in goals) g.initialise();
		}

		/** Is the level complete */
		public function get complete(): Boolean {
			for each(var g:Goal in goals) if(!g.reached) return false;
			return true;
		}

		/** Collide a shape with everything */
		public function collide(s:Shape, x:Number, y:Number): Boolean {
			// Test all walls
			if(level.collide(x, y, Shape.radius)) return true;
			// Test other objects
			for each(var o:GameObject in collidable) if(o.collideShape(s,x,y)) return true;
			return false;
		}

		/** Update loop */
		public override function update(): void {
			super.update();
			
			// Directoinal input
			if(Input.mousePressed && state!=WIN) {
				selected = null;
				for each(var s:Shape in shapes) {
					if(!s.moving && s.distanceToPoint(mouseX, mouseY) < Shape.radius) selected = s;
				}
				if(selected) trace("Selected shape", selected.symbol);
			} else if(selected) {
				// Direction arrow
				if(selected.distanceToPoint(mouseX, mouseY) < Shape.radius) direction = 0;
				else {
					// Get movement direction
					var dx: Number = mouseX - selected.x;
					var dy: Number = mouseY - selected.y;
					direction = Math.abs(dx) > Math.abs(dy)? 2: 1;
					if((direction==1 && dy>0) || (direction==2 && dx<0)) direction+=2;
					if(!selected.canMove(direction)) direction = 0;

					// Direction arrow
					if(direction) {
						arrow.x = selected.x;
						arrow.y = selected.y;
						(arrow.graphic as Image).angle = (direction-1) * -90;
					}
				}

				// MouseUp - Move
				if(!Input.mouseDown) {
					trace("Move shape", direction);
					if(direction) {
						selected.move(direction);
						Audio.play("slide");
						++moves;
						moveText.text = "Moves\n" + moves + " / " + level.target;

					}
					selected = null;
				}

				// show arrow
				arrow.visible = selected && direction>0;
			}

			// Win state
			if(state==WIN) {
				if(winValue<1) {
					winValue = Math.min(1.0, winValue+0.03);
					winText.alpha = winValue;
					FP.rect.x = FP.rect.y = 0; 
					FP.rect.width = winOverlay.width;
					FP.rect.height = winOverlay.height;
					winOverlay.fillRect(FP.rect, 0xffffff | (uint)(winValue*180)<<24);
					if(winValue==1.0) winButton.visible = true;
				}
			} else if(complete) win();


			// Shortcut keys
			if(Input.pressed(Key.R)) restart();
			if(Input.pressed(Key.ESCAPE)) goBack();

			CONFIG::DEBUG {
				if(Input.pressed(Key.C)) win();

				// Editing
				if(Input.pressed(Key.E)) editing = !editing;
				if(editing) {
					var r:Number = Shape.radius;
					var mx:Number = Math.round(mouseX/r)*r;
					var my:Number = Math.round(mouseY/r)*r;
					if(Input.mousePressed && !selected) {
						edit = { x:mx, y:my };
					} else if(edit && Input.mouseReleased) {
						if(edit.x!=mx || edit.y!=my) {
							level.addLine(edit.x, edit.y, mx, my);
							walls.graphic = level.wallGraphic();
						}
						edit = null;
					}
					if(Input.pressed(Key.A)) {
						shapes.push( new Shape(mx, my, 5) );
						add( shapes[shapes.length-1] );
						collidable.push( shapes[shapes.length-1] );
					}
					if(Input.pressed(Key.UP))    collidable.push( add( new Push(mx, my, 1) ) );
					if(Input.pressed(Key.RIGHT)) collidable.push( add( new Push(mx, my, 2) ) );
					if(Input.pressed(Key.DOWN))  collidable.push( add( new Push(mx, my, 3) ) );
					if(Input.pressed(Key.LEFT))  collidable.push( add( new Push(mx, my, 4) ) );
					if(Input.pressed(Key.K)) {
						level.lines = [];
						walls.graphic = level.wallGraphic();
						var p:Array = []; getClass(Push, p); removeList(p);
						removeList(shapes); shapes = []; collidable=[];
					}
					if(Input.pressed(Key.U)) {
						level.lines.pop();
						walls.graphic = level.wallGraphic();
					}
					if(Input.pressed(Key.P)) {
						r = Shape.radius * 2;
						trace("----------------------------");
						for each(var w:Object in level.lines) trace("line", w.x0/r, w.y0/r, w.x1/r, w.y1/r);
						for each(var sh:Shape in shapes) trace("shape", sh.symbol, sh.x/r-0.5, sh.y/r-0.5);
						for each(var gl:Goal in goals) trace("goal", gl.symbol, gl.x/r-0.5, gl.y/r-0.5);
						trace("----------------------------");
					}
				}
			}
		}

		CONFIG::DEBUG {
			public override function render(): void {
				super.render();
				if(editing) {
					var r:Number = Shape.radius;
					// grid
					for(var i:int=0; i<=10; ++i) {
						Draw.line(i*r*2, 0, i*r*2, 10*r*2, 0xe0e0e0);
						Draw.line(0, i*r*2, 10*r*2, i*r*2, 0xe0e0e0);
					}
					if(edit) {
						edit.x2 = Math.round(mouseX/r)*r;
						edit.y2 = Math.round(mouseY/r)*r;
						Draw.line(edit.x, edit.y, edit.x2, edit.y2, 0x00);
					}
				}
			}
		}


	}
}

