package {

	import net.flashpunk.*;
	import net.flashpunk.graphics.*;

	public class Goal extends GameObject {
		
		public var symbol: int;
		public var filled: Shape = null;
		
		public function Goal(x:Number, y:Number, type:int) {
			super(x, y);
			var img:Image = Shape.createGraphic(type, Shape.radius);
			img.alpha = 0.3;
			graphic = img;
			symbol = type;
		}

		/** Calculate initial state */
		public function initialise(): void {
			filled = null;
			for each(var s:Shape in (world as Game).shapes) {
				if(s.symbol == symbol && s.distanceFrom(this)<4) filled = s;
			}
		}

		/** Is this goal reached? */
		public function get reached(): Boolean {
			return filled && !filled.moving;
		}

		/** Collision test */
		public override function collideShape(s:Shape, x:Number, y:Number): Boolean {
			var f:Boolean = s.symbol==symbol && distanceToPoint(x,y)<4;
			// Emit some particles?
			if(f && !filled) filled = s;
			else if(filled==s && !f) filled = null;
			return false;
		}

		/** Goal event */
		public function goalEvent(): void {
			Audio.play("goal");
			world.add( new Ripple(x, y) );
			trace("Success!");
		}

		/** Fading */
		public override function update(): void {
			var g:Image = graphic as Image;
			if(filled && filled.moving) filled = null;
			else if(filled && g.alpha==0.3) goalEvent();
			if(filled && g.alpha<1) g.alpha = Math.min(1.0, g.alpha+0.01); 
			else if(!filled && g.alpha>0.3) g.alpha = Math.max(0.3, g.alpha-0.01);
		}

	}

}
