package {

	import net.flashpunk.FP;
	import net.flashpunk.utils.Data;
	import net.flashpunk.graphics.Stamp;
	import flash.display.BitmapData;
	import flash.display.Graphics;
	import flash.geom.Rectangle;
	import flash.geom.Matrix;

	public class Level {
		
		[Embed(source="levels.txt", mimeType="application/octet-stream")]	public static var levels_src: Class;
		public static var list: Array = []; // List of all levels

		private static const lineWidth: Number = 4.0;
		private static const lineTaper: Number = 30.0;
		private static const lineEnd:   Number = 0;

		
		public var lines: Array;	// List of all lines
		public var rect:Rectangle;	// Level bounds
		public var target: int;		// Target move count
		public var objectData: Array;


		public var score: int = 0;	// User score
		public var locked: Boolean;	// Level is locked
		public var id: String;		// Level id

		/** Load a list of levels from a string 
			Positions should be multiples of shape size
		*/
		public static function loadList(source:String): int {
			var m: Number = Shape.radius * 2;	// Position multiplier
			var o: Number = Shape.radius;		// Object offset (for centred objects)
			var startCount:int = list.length;
			var levelString:Array = source.split(/\[(.*)\]/);
			for(var i:int=1; i+1<levelString.length; i+=2) {
				trace("Load level", levelString[i]);
				if(levelString[i].length>0 && levelString[i+1].length>10) {
					// Load level
					var l:Level = new Level(levelString[i]);
					var lines: Array = levelString[i+1].split("\n");
					for each(var s:String in lines) {
							var parts:Array = s.split(/[ \t]+/);
							switch(parts[0]) {
							case "": break;
							case "target": l.target = parts[1]; break;
							case "line":   l.addLine(parts[1]*m, parts[2]*m, parts[3]*m, parts[4]*m); break;
							case "box":    l.addBox(parts[1]*m, parts[2]*m, parts[3]*m, parts[4]*m); break; // Shorthand for 4 lines
							case "shape":  l.objectData.push( { type:Shape, value:parts[1], x:parts[2]*m+o, y:parts[3]*m+o } ); break;
							case "goal":   l.objectData.push( { type:Goal,  value:parts[1], x:parts[2]*m+o, y:parts[3]*m+o } ); break;
							case "push":   l.objectData.push( { type:Push,  value:parts[1], x:parts[2]*m+o, y:parts[3]*m+o } ); break;
							default: if(parts[0].charAt(0)!='#') trace("Undefined type", parts[0]);
						}
					}
					// Add to list
					if(l.lines.length>0) {
						l.locked = l.score==0 && list.length>0 && list[list.length-1].score==0;
						list.push(l);
					}
				}
			}
			return list.length - startCount;
		}

		
		/** Level constructor */
		public function Level(id:String) {
			this.id = id;
			rect = new Rectangle();
			objectData = [];
			lines = [];
			// Load local data
			score = Data.readInt("s"+id, 0);
		}

		/** Add a line to the level */
		public function addLine(x0: Number, y0: Number, x1:Number, y1:Number): void {
			var l:Object = { x0:x0, y0:y0, x1:x1, y1:y1 };
			lines.push(l);
			// Update rect
			if(rect.width==0 && rect.height==0) rect.offset(x0, y0);
			rect.left   = Math.min(rect.left,   x0-6, x1-6);
			rect.right  = Math.max(rect.right,  x0+6, x1+6);
			rect.top    = Math.min(rect.top,    y0-6, y1-6);
			rect.bottom = Math.max(rect.bottom, y0+6, y1+6);
		}

		/** Add four lines to make a box */
		public function addBox(x:Number, y:Number, w:Number, h:Number): void {
			addLine(x, y, x+w, y);
			addLine(x, y, x, y+h);
			addLine(x+w, y, x+w, y+h);
			addLine(x, y+h, x+w, y+h);
		}

		/** Create level objects */
		public function get objects(): Array {
			var obj:Array = [];
			for each(var o:Object in objectData) {
				obj.push( new o.type(o.x, o.y, o.value) );
			}
			return obj;
		}

		/** Save local data */
		public function saveData(): void {
			Data.writeInt("s"+id, score);
			Data.save("kriok");
		}

		/** Get level index */
		public function get index(): int {
			return list.indexOf(this);
		}

		/** Get the next level */
		public function get next(): Level {
			var i:int = index;
			if(i>=0 && i < list.length-1) return list[i+1];
			else return null;
		}


		/** Test collisions with walls */
		public function collide(x:Number, y:Number, r:Number, skip:Object=null): Boolean {
			for each(var l:Object in lines) {
				if(l!=skip) {
					// Closest point on line is simplest - actually isn't as lines are axis aligned but meh
					var dx:Number = l.x1 - l.x0;
					var dy:Number = l.y1 - l.y0;
					var sx:Number = x - l.x0;
					var sy:Number = y - l.y0;
					var t:Number = FP.clamp((dx*sx + dy*sy) / (dx*dx + dy*dy), 0, 1);
					var d:Number = FP.distance(x, y, l.x0+dx*t, l.y0+dy*t);
					if(d<r) return true;
				}
			}
			return false;
		}


		/** Build a graphic from the wall information */
		public function wallGraphic(scale:Number=1.0): Stamp {
			FP.sprite.graphics.clear();
			FP.sprite.graphics.lineStyle(lineWidth, 0x00, 1.0, true, "normal", "square", "miter", 16);
			for each(var l:Object in lines) drawLine(l);
			var bmp:BitmapData = new BitmapData(rect.width*scale, rect.height*scale);
			var matrix:Matrix = new Matrix(scale, 0, 0, scale, -rect.x, -rect.y);
			bmp.draw(FP.sprite, matrix, null, null); // One of these is scale
			return new Stamp(bmp, rect.x, rect.y);
		}

		/** Draw a line to the level sprite */
		private function drawLine(l:Object): void {
			// Do the cool tapering later.
			var g:Graphics = FP.sprite.graphics;
			var w:Number = lineWidth/2;
			//g.beginFill(0,1);
			//if(l.x0==l.x1)      g.drawRect(l.x0-w, l.y0-w, lineWidth, l.y1-l.y0+lineWidth);
			//else if(l.y0==l.y1) g.drawRect(l.x0-w, l.y0-w, l.x1-l.x0+lineWidth, lineWidth);
			//g.endFill();

			g.moveTo(l.x0, l.y0);
			g.lineTo(l.x1, l.y1);
		}
	}

}
