package {

	import net.flashpunk.*;
	import net.flashpunk.utils.*;
	import net.flashpunk.graphics.*;

	public class LevelSelect extends World {

		public static var levels: Array; // Level list

		public function LevelSelect() {
			// Title
			FP.screen.color = 0xffffff;
			var ts:Object = { font:"brush", size:32, color:0x00 };
			addGraphic( new Text("Level Select", 10, 10, ts) );

			// Ambiance
			var shape:Image = Shape.createGraphic(2, 200);
			shape.tintMode = 1.0;
			shape.color = 0xa0a0a0;
			addGraphic(shape, 0, 450, 100);


			// Back button
			add( new Button("Back", FP.halfWidth-50, FP.height-50, 100, 40, goBack) );

			// Cache shape images
			var shapes:Array = [];
			for(var i:int=0; i<6; ++i) {
				shapes[i] = Shape.createGraphic(i, 20);
				shapes[i].x = shapes[i].y = 5;
			}
			
			// Create buttons for each level
			var ss:Object = { font:"brush", size:16, color:0x00, width:50, align:"center" };
			for(var l:int=0; l<Level.list.length; ++l) {
				var level:Level = Level.list[l];
				var x: Number = (l%5) * 100 + 75;
				var y: Number = Math.floor(l/5)*100 + 100;

				if(level.locked) {
					addGraphic( shapes[l%6], 0, x, y);
				} else {
					// Create button
					var b:Button = new Button(null, x,y, 50, 50, start);
					b.addGraphic( shapes[l%6] );
					add(b);

					// Score
					if(level.score>0) {
						var score:String = level.score + " / " + level.target;
						addGraphic( new Text(score, 0,0,ss), 0, b.x, b.y + b.height );
					}
				}
			}
		}

		/** Back to previous menu */
		public function goBack(): void {
			Transition.fadeOut( new Title(), 0xffffff );
			Audio.play("change");
		}

		/** Start a level */
		public function start(): void {
			// Which button was pressed?
			var i:int = Math.round(mouseX/100) + Math.round(mouseY/100) * 5 - 6;
			trace("Start level", i);
			Transition.fadeOut( new Game( Level.list[i], 1 ), 0xffffff );
			Audio.play("change");
		}

		/** Update - for keys */
		public override function update(): void {
			super.update();
			if(Input.pressed(Key.ESCAPE)) goBack();
			CONFIG::DEBUG {
				if(Input.pressed(Key.U)) {
					for each(var l:Level in Level.list) l.locked = false;
					FP.world = new LevelSelect();
				}
			}
		}

	}
}
