package {

	import net.flashpunk.*;
	import net.flashpunk.graphics.Image;
	import flash.display.BitmapData;
	import flash.display.Graphics;
	
	public class Shape extends GameObject {

		public static const radius: Number = 20;
		public static var speed: Number = 6;
		public var symbol: int;

		public var mx:Number=0, my:Number=0; // Movement
		
		public function Shape(x:Number, y:Number, type:int) {
			super(x, y, createGraphic(type, radius));
			symbol = type;
			setHitboxTo(graphic);
			layer = -1;
		}

		public static function createGraphic(type:int, radius:Number): Image {
			const r: Number = radius * 0.5;
			const c: Number = radius;
			var g:Graphics = FP.sprite.graphics;
			// Circle
			g.clear();
			g.lineStyle(c-r, 0x00, 1.0);
			g.drawCircle(c,c, c-r/2);
			// Symbol
			const t: Number = Math.PI*2;
			const pc: Array = [ 3,   3,   4,   4,   6,   5 ];		// Point count
			const ps: Array = [ t/3, t/3, t/4, t/4, t/6, t/5*2 ];	// Angle step
			const po: Array = [ 0,   t/2, 0,   t/8, 0,   0 ];		// Angle offset
			g.lineStyle(radius * 0.1, 0x00, 1.0, false, "normal");
			var a:Number = po[type];
			g.moveTo( c+Math.sin(a)*r, c+Math.cos(a)*r);
			for(var i:int=0; i<=pc[type]; ++i) {
				a += ps[type];
				g.lineTo( c+Math.sin(a)*r, c+Math.cos(a)*r);
			}
			// Make image
			var bmp:BitmapData = new BitmapData(radius*2, radius*2, true, 0);
			bmp.draw(FP.sprite, null, null, null);
			var img:Image = new Image(bmp);
			img.x = img.y = -radius;
			return img;
		}

		/** Shape collision test */
		public override function collideShape(s:Shape, x:Number, y:Number): Boolean {
			return this!=s && distanceToPoint(x,y)<2*radius;
		}

		/** Can this shape move in this direction */
		public function canMove(direction:int): Boolean {
			if(moving) return false;
			move(direction);
			var can:Boolean = (world as Game).collide(this, x+mx, y+my) == false;
			move(0);
			return can;
		}
		/** Start shape moving */
		public function move(direction:int): void {
			switch(direction) {
			case 1: mx = 0; my =-1; break;
			case 2: mx = 1; my = 0; break;
			case 3: mx = 0; my = 1; break;
			case 4: mx =-1; my = 0; break;
			default: mx = my = 0; break;
			}
		}
		/** Is this shape moving */
		public function get moving(): Boolean {
			return mx!=0 || my!=0;
		}
		/** Update movement */
		public override function update(): void {
			var game:Game = world as Game;
			if(moving) {
				// Need to do collision
				if(game.collide(this, x+mx*speed, y+my*speed)) {
					for(var i:Number=1; i<speed; ++i) {
						if(game.collide(this, x+mx*i, y+my*i)) break;
					}
					x += mx*(i-1);
					y += my*(i-1);
					move(0); // Stop moving
					// Check goals incase they were missed
					for each(var g:Goal in game.goals) g.collideShape(this, x, y);
				} else {
					x += mx*speed;
					y += my*speed;
				}
			}
		}
	}
}
